<?php

    function idea_push_list_items($boardId,$order,$status,$tag){
        
        //get all posts
        global $post;
        
        //get options
        $options = get_option('idea_push_settings');


        $individualBoardSetting = idea_push_get_board_settings($boardId);
        $multiIp = $individualBoardSetting[27];

        if(!isset($multiIp)){
            $multiIp = 'No';
        }
    
        //get current user
        $currentUser = idea_push_check_if_non_logged_in_user_is_guest($multiIp);
        
        //get board settings
        $boardSettings = idea_push_get_board_settings($boardId);
        
        $paginationNumber = $boardSettings[15];
        
        //this is used for pagination
        $ideaNumber = 1;
        
        //check if pagination required
        if(isset($paginationNumber) && $paginationNumber>0){
            $paginationEnabled = true;
        } else {
            $paginationEnabled = false;    
        }


        //create html holding variable that is returned at the end
        $html = '';


        //so we have the variables order, status and tag that we need to contend with, the board id is set in stone
        //so lets start building our query

        //this is the base query as it should be common for all
        $args = array(
            'suppress_filters' => 0,
            'post_type' => 'idea',
            'post_status' => 'publish',
            'posts_per_page' => -1,
            'tax_query' => array("relation" => "AND",
            array(
                'taxonomy' => 'boards',
                'field'    => 'term_id',
                'terms'    => array($boardId)
            )),
        );

        // var_dump($args);


        // echo '<strong>order is:'.$order.' status is:'.$status.' tag is:'.$tag.'</strong>';

        //lets start with the status component of the query
        //if the status is not show all statuses then lets add the status argument to the query
        if($status !== 'all-statuses'){

            //get original value
            $originalValue = $args['tax_query'];

            //length of array
            $lengthOfArray = count($originalValue);

            $originalValue[$lengthOfArray-1] = array(
                'taxonomy' => 'status',
                'field'    => 'slug',
                'terms'    => array($status)
            );

            $args['tax_query'] = $originalValue;
        }




        //now lets deal with the tags
        if($tag !== 'all'){
            
            //get original value
            $originalValue = $args['tax_query'];

            //length of array
            $lengthOfArray = count($originalValue);

            //now lets add our array to the end
            $originalValue[$lengthOfArray-1] = array(
                'taxonomy' => 'tags',
                'field'    => 'term_id',
                'terms'    => array($tag)
            );

            $args['tax_query'] = $originalValue;
        }




        //now we have to deal with the order
        //lets deal with this as a switch statement
        switch ($order) {

            case "popular":
                $args['orderby'] = 'meta_value_num';
                $args['order'] = 'DESC';
                $args['meta_key'] = 'votes';
                break;

            case "recent":
                $args['orderby'] = 'publish_date';
                $args['order'] = 'DESC';
                $args['meta_key'] = '';
                break;

            case "trending":
            
                //we need to get all posts and loop through them, and for each we need to get the date published and work out the amount of days between them and now and divide this by the votes and then update the post meta
                
                //we can use our existing argument array even though it's not fully finished at this point.
                $ideaPostsTrending = get_posts($args);
                
                $todaysDate = new DateTime('now');
                
                foreach($ideaPostsTrending as $ideaPostTrending){
                    
                    $postDate = new DateTime($ideaPostTrending->post_date);
                    
                    $dateDifference = $postDate->diff($todaysDate);
                    
                    //we add plus one do division isn't done by 0 if the post was published today
                    $daysDifference = $dateDifference->format('%a')+1;
                    
                    //get amount of votes
                    $amountOfVotes = get_post_meta($ideaPostTrending->ID, 'votes', true );
                    
                    if(empty($amountOfVotes)){
                        $amountOfVotes = 0;
                        
                        //for good measure lets create a votes score - maybe if someone was to manually delete it from the post or something - but it really show be created when first creating the post via the form
                        update_post_meta($post_id = $ideaPostTrending->ID, $key = 'votes', $value = 0);
                        
                    }
                    
                    if($amountOfVotes == 0){
                        $trendScore = 0;
                    } else {
                        $trendScore = $amountOfVotes/$daysDifference;   
                    }
                    
                    //lets now update the meta
                    update_post_meta( $post_id = $ideaPostTrending->ID, $key = 'trendscore', $value = $trendScore);
                    
                }

                $args['orderby'] = 'meta_value';
                $args['order'] = 'DESC';
                $args['meta_key'] = 'trendscore';
                break;

            case "my-voted":
                $args['orderby'] = 'publish_date';
                $args['order'] = 'DESC';
                $args['meta_key'] = '';
                break;    
            
            //its "my" order query
            default:
                $args['orderby'] = 'publish_date';
                $args['order'] = 'DESC';
                $args['meta_key'] = '';
                $args['author'] = $order;

        }

        //lets print the array for testing
        // var_dump($args);
        // print_r($args);

        //run the query
        $ideaPosts = get_posts($args);
        
        //if there are no posts show an error
        if(empty($ideaPosts)){
            return '<li class="idea-item no-ideas-message">'.__('No ideas found matching this criteria','ideapush').'</li>';    
        }
        
        

        /**
        * 
        *
        *
        * Start looping through each idea
        */
        foreach($ideaPosts as $ideaPost){
        
             
            //if the order is set to my-voted then check to see if current user is a positive voter of the id    
            if($order=='my-voted'){

                //get positive voters of idea
                $positiveVoters = get_post_meta($ideaPost->ID,'up-voters',true);

                $positiveVotersArray = array();

                foreach($positiveVoters as $voter){
                    array_push($positiveVotersArray,$voter[0]);
                }


                //get current user id
                $currentUserId = idea_push_check_if_non_logged_in_user_is_guest($multiIp);

                if(!in_array($currentUserId,$positiveVotersArray)){

                    continue;    
                } 
            }    
            
       
            if($paginationEnabled){
                $html .= '<li data-page="'.ceil($ideaNumber/$paginationNumber).'" class="idea-item idea-item-'.$ideaPost->ID.' hidden-idea">';    
            } else {
                $html .= '<li class="idea-item idea-item-'.$ideaPost->ID.'">';    
            }    
            
            

                //we need to rename the status variable to the status of the post as oppose to the status sent to the function as we are now enabling all ideas
                $status = get_post_meta($ideaPost->ID,'current-status',true);


                if(!isset($status) || strlen($status)<1){

                    //get current status
                    $currentStatus = get_the_terms( $ideaPost->ID, 'status' );

                    $status = $currentStatus[0]->slug;

                    //update the post meta for ideas created in the backend
                    update_post_meta($ideaPost->ID, 'current-status', $status );

                }

                

                //left side
                $html .= '<div class="idea-item-left">';

                    //do votes
                    $html .= idea_push_vote_part($ideaPost->ID,$status,$boardSettings);

                $html .= '</div>'; //end left side
                
            
                //right side
                $html .= '<div class="idea-item-right">';
                
                    $html .= '<div class="idea-item-right-inner">';
                        
            
                            
                    
                        //title
                        //if pro option is set
                        if(!isset($options['idea_push_disable_single_idea_page']) || $options['idea_push_disable_single_idea_page'] !== "1"){
                            $html .= '<a href="'.esc_url(get_post_permalink($ideaPost->ID)).'" class="idea-title">';
                        } else {
                             $html .= '<a class="idea-title">';    
                        }
            
                        $html .= esc_html($ideaPost->post_title);
                        $html .= '</a>';    


                        //get the meta
                        $html .= idea_push_get_idea_meta($ideaPost,$status,$boardSettings);
                        
                
                        
                        //get the content
                        $html .= '<div class="idea-item-content-container">';
                        
                        if(!isset($options['idea_push_disable_single_idea_page']) || $options['idea_push_disable_single_idea_page'] !== "1"){
                             
                            if(strlen($ideaPost->post_content)>200){
                                $readMore = '... <a class="idea-read-more" href="'.esc_url(get_post_permalink($ideaPost->ID)).'">'.__('Read more','ideapush').'...</a>';
                            } else {
                                $readMore = '';   
                            }
    
                            $html .= '<span class="idea-item-content">'.mb_substr(strip_tags($ideaPost->post_content),0,200).'</span>'.$readMore;     
                        } else {
                            
                            $html .= '<div id="ideaReadMoreText" style="display: none;">'.__('Read more','ideapush').'</div>';
                            
                            $html .= '<div id="ideaReadLessText" style="display: none;">'.__('Read less','ideapush').'</div>';
                            
                            $html .= '<span class="idea-item-content idea-item-content-read-more">'.strip_tags($ideaPost->post_content).'</span>';    
                            
                        }
                        
            
                        $html .= '</div>';
    
                
                        //get attachment and tags
                        $html .=  idea_push_get_tags_and_attachments($ideaPost->ID);
                    
                    $html .= '</div>';
                $html .= '</div>';
            
            $html .= '</li>';  
            
            
            //add one to the idea number
            $ideaNumber++;
            
        } //end loop for each post idea
        



        /**
        * 
        *
        *
        * Do pagination
        */
        if($paginationEnabled){
            
            $amountOfPages = ceil(($ideaNumber-1)/$paginationNumber);
            
            $html .= '<li class="idea-pagination">';
            
                $html .= '<ul class="idea-pagination-listing">';
            
                    for($i=1;$i<=$amountOfPages;$i++){
                        
                        if($i == 1){
                            
                            $html .= '<li class="idea-page-number active idea-page-'.$i.'">'.$i.'</li>';    
                        } else {
                            
                            $html .= '<li class="idea-page-number idea-page-'.$i.'">'.$i.'</li>';    
                        }
                        
                            
                        
                    }
                
                $html .= '</ul>';
    
            $html .= '</li>';    
            
        }
  
        return $html;  
    }

    


    function idea_push_fetch_new_list_items(){

        $boardNumber = idea_push_sanitization_validation($_POST['boardNumber'],'id');


        if($boardNumber == false){
            wp_die();      
        }
        
        $sortFilter = $_POST['sortFilter'];
        $statusFilter = $_POST['statusFilter'];
        $tagFilter = $_POST['tagFilter'];

        echo idea_push_list_items($boardNumber,$sortFilter,$statusFilter,$tagFilter);
        
        wp_die(); 

    } //end function
    add_action( 'wp_ajax_get_new_ideas', 'idea_push_fetch_new_list_items' );
    add_action( 'wp_ajax_nopriv_get_new_ideas', 'idea_push_fetch_new_list_items' );

    

    function idea_push_get_tags_and_attachments($postId){
        
        $html = '';
        $html .= '<div class="idea-item-additional-meta">';


            //do attachments
            if(get_the_post_thumbnail_url($postId) !== false){

                $html .= '<span class="idea-item-attachment"><i class="fa fa-picture-o" aria-hidden="true"></i>';

                $html .= '<a target="_blank" href="'.esc_url(get_the_post_thumbnail_url($postId)).'" class="idea-item-file">'.__( 'Attachment','ideapush' ).'</a>';   

                $html .= '</span>';    

            }



            //do files
            if( strlen(get_post_meta($postId,'idea-attachment',true))>0 ){

                $ideaAttachment =  get_post_meta($postId,'idea-attachment',true);

                $html .= '<span class="idea-item-attachment-file"><i class="fa fa-file" aria-hidden="true"></i>';

                $html .= '<a target="_blank" href="'.esc_url($ideaAttachment).'" class="idea-item-file-download">'.__( 'File','ideapush' ).'</a>';   

                $html .= '</span>';    

            }



            //do tags
            $ideaTags = get_the_terms($postId,'tags');
            //check to see if tags exist
            if(!empty($ideaTags)){    

                $html .= '<span class="idea-item-tags"><i class="fa fa-tags" aria-hidden="true"></i> ';

                foreach($ideaTags as $ideaTag){

                    if(strpos($ideaTag->name, 'BoardTag-') !== false){

                        $positionOfSecondHyphen = strpos($ideaTag->name, '-', strpos($ideaTag->name, '-') + 1);

                        $tagName = substr($ideaTag->name,$positionOfSecondHyphen+1,strlen($ideaTag->name)-$positionOfSecondHyphen);

                    } else {
                        $tagName = $ideaTag->name;
                    }
                    
                    $html .= '<a href="'.esc_url(get_tag_link($ideaTag->term_id)).'" class="idea-item-tag">'.esc_html($tagName).'</a>';    
                }    

                $html .= '</span>';
            }


            //do custom fields
            //only do if pro and it is enabled in the settings do it
            global $ideapush_is_pro;
            $options = get_option('idea_push_settings');

            if($ideapush_is_pro == "YES" && isset($options['idea_push_show_custom_fields'])){

                $postMeta = get_post_meta($postId,'',true);

                $customFieldBuilder = '';

                foreach($postMeta as $key => $value){

                    //check if key has our special prefix
                    if(strpos($key, 'ideapush-custom-field-') !== false) {

                        $customFieldBuilder .= '<strong>'.str_replace("ideapush-custom-field-","",$key).':</strong> ';
                        $customFieldBuilder .= $value[0].', ';

                    }    

                } 

                if(strlen($customFieldBuilder) > 0){
                    $html .= '<span class="idea-item-custom-fields"><i class="fa fa-list" aria-hidden="true"></i> ';
                }
                
                $customFieldBuilder = rtrim($customFieldBuilder,', ');

                $html .= '<span class="custom-field-content">'.$customFieldBuilder.'</span>';

                $html .= '</span>';


            }



        
        $html .= '</div>';
        return $html;

    }





    function idea_push_get_idea_meta($ideaPost,$status,$boardSettings){
        
        //get options
        $options = get_option('idea_push_settings');
        
        $html = '';
        
        $html .= '<div class="idea-item-meta">';
                        
        //date published
        $html .= '<span class="idea-item-date">'.esc_html(human_time_diff(strtotime($ideaPost->post_date), current_time( 'timestamp' ) )).' '.__( 'ago','ideapush' ).'</span>';

        //author
        $html .= '<div class="idea-author-container">';
            $html .= '<img class="idea-author-profile-image" src="'.esc_url(idea_push_get_user_avatar(get_the_author_meta('ID',$ideaPost->post_author))).'" />';
        
            //do admin star            
            if(idea_push_is_user_admin(get_the_author_meta('ID',$ideaPost->post_author))){
                $html .= '<span class="admin-star-outer"><i class="fa fa-star admin-star-icon" aria-hidden="true"></i></span>';    
            }

            $html .= '<a href="'.esc_url(idea_push_get_user_author_page(get_the_author_meta('ID',$ideaPost->post_author))).'" class="idea-author">'.esc_html(idea_push_get_user_name(get_the_author_meta('ID',$ideaPost->post_author))).'</a>';
        $html .= '</div>';

        //status
        //here we need to change the status to the translated status, a little painful but yeah
        
        $html .= '<span class="status-container">'.idea_push_render_status($status,'ONLY').'</span>';


        //comments
        //we only want to show comments if comments are enabled for this board                    
        $showComments = $boardSettings[4];

        if($showComments == 'Yes'){

            //change class if no comments
            if($ideaPost->comment_count == 0){
                $commentClass = 'no-comments-idea';
            } else {
                $commentClass = '';    
            }

            //comment link
            global $wp;
            $currentPageUrl = home_url( $wp->request ).'/';

            // var_dump(get_post_permalink($ideaPost->ID));
            // var_dump($currentPageUrl);

            if($currentPageUrl == get_post_permalink($ideaPost->ID)){
                $commentLink = '#comments';       
            } else {
                $commentLink = esc_url(get_post_permalink($ideaPost->ID)).'#comments';   
            }
            
            
            if(!isset($options['idea_push_disable_single_idea_page']) || $options['idea_push_disable_single_idea_page'] !== "1"){
                $html .= '<a class="idea-item-comments '.esc_html($commentClass).'" href="'.esc_url($commentLink).'">';  
            } else {
                $html .= '<a class="idea-item-comments '.esc_html($commentClass).'" >';     
            }
            
            
            $html .= esc_html($ideaPost->comment_count).'<i class="fa fa-comment-o comments-icon" aria-hidden="true"></i>';
            
            
            $html .= '</a>';    
           
            
        }

        global $ideapush_is_pro;

        if($ideapush_is_pro == 'YES'){
            //do user edit/delete if enabled
            if(isset($boardSettings[25]) && $boardSettings[25] == 'Yes'){

                //only show if the user has permission to do the action
                if(get_the_author_meta('ID',$ideaPost->post_author) == get_current_user_id()){
                    $html .= idea_push_user_edit_delete($ideaPost->ID);
                }

            }

            //if the user is admin also enable duplicate feature
            if( current_user_can('editor') || current_user_can('administrator') ) {
                $html .= idea_push_admin_duplicate($ideaPost->ID);
            }
            

        }

        


        $html .= '</div>';

        return $html;
        
        
    }



function idea_push_vote_part($postId,$status,$boardSettings){
    
    if(!isset($boardSettings[27])){
        $multiIp = 'No';
    } else {
        $multiIp = $boardSettings[27];    
    }

    //get current user
    $currentUser = idea_push_check_if_non_logged_in_user_is_guest($multiIp);
    
    $html = '';
    
    $html .= '<div class="idea-vote-container" data="'.esc_attr($postId).'">';

        $voteAmount = get_post_meta($postId,'votes',true);

        if(!isset($voteAmount) || strlen($voteAmount) == 0){
            $voteAmount = 0;
        }

    
        $html .= '<span class="idea-vote-number">'.esc_html($voteAmount).'</span>';

        //we only want to show voting for open statuses
        if($status == 'open'){
            
            

            //now we need to check whether guest voting is enabled
            if( $boardSettings[9] == "Yes" || $currentUser !== false ){
                
                global $ideapush_is_pro;
                
                //only get the following settings if pro
                if($ideapush_is_pro == 'YES'){
                    $enableChallenge = $boardSettings[16];
                    $challengeDate = $boardSettings[18];
                    $challengeTime = $boardSettings[19];
                }

                //do challenge check
                if(($ideapush_is_pro == 'NO') || (!isset($enableChallenge)) || ($ideapush_is_pro == 'YES' && $enableChallenge !== 'Yes') || ($ideapush_is_pro == 'YES' && $enableChallenge == 'Yes' && strtotime(current_time('mysql')) < strtotime($challengeDate.' '.$challengeTime.':00'))){
               
                    $downVotersArray = idea_push_get_voter_array($postId,'down-voters');
                    $upVotersArray = idea_push_get_voter_array($postId,'up-voters');

                    if(idea_push_check_voter_history($upVotersArray,$currentUser) && $currentUser !== false){
                        $html .= '<i class="fa fa-thumbs-up vote-up-unvoted" aria-hidden="true"></i>';  
                    } else {
                        $html .= '<i class="fa fa-thumbs-o-up vote-up-unvoted" aria-hidden="true"></i>';     
                    }


                    $downVoting = $boardSettings[10];

                    if($downVoting == 'Yes'){



                        if(idea_push_check_voter_history($downVotersArray,$currentUser) && $currentUser !== false){
                            $html .= '<i class="fa fa-thumbs-down vote-down-unvoted" aria-hidden="true"></i>';   

                        } else {
                            $html .= '<i class="fa fa-thumbs-o-down vote-down-unvoted" aria-hidden="true"></i>';
                        }

                    }  
                
                
                } //end challenge check
                
                
                

            } //end guest voting check

        } //end open check

    

    $html .= '</div>'; //end vote container
    
    return $html;
    
}



function idea_push_update_vote_counter(){
    
    $ideaId = idea_push_sanitization_validation($_POST['ideaId'],'id');
    $status = sanitize_text_field($_POST['status']);
    
    if($ideaId == false){
        wp_die();    
    }
    
    
    
    $boardSettings = idea_push_get_board_settings(idea_push_get_board_id_from_post_id($ideaId));
    
    
    echo idea_push_vote_part($ideaId,$status,$boardSettings);
   
    wp_die();    
}

add_action( 'wp_ajax_update_vote_counter', 'idea_push_update_vote_counter' );
add_action( 'wp_ajax_nopriv_update_vote_counter', 'idea_push_update_vote_counter' );



?>